
/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log$
*/

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  TOF Cable Length Map class                                               //
//                                                                           //
//                                                                           //
//  Author: P.Pagano                                                         //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#include "AliTOFCableLengthMap.h"

ClassImp(AliTOFCableLengthMap)

const Float_t AliTOFCableLengthMap::fgkPropagationDelay = 0.0513;

const Float_t AliTOFCableLengthMap::fgkCableLength[72][10][2][5] =
{
  {//crate 0
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,387.5,387.5,400.0},{412.5,412.5,425.0,412.5,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,425.0,425.0},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,475.0,475.0,475.0,475.0},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,550.0,550.0}} //slot 12, chain
  },
  {//crate 1
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,400.0,400.0,400.0}},//slot 8, chain  
    {{412.5,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,437.5,437.5,437.5},{475.0,475.0,462.5,450.0,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,475.0,475.0},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.0,525.0,512.5,512.5,512.5},{550.0,550.0,550.0,537.5,537.5}} //slot 12, chain
  },
  {//crate 2
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,387.5,387.5,400.0},{412.5,412.5,425.0,412.5,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,425.0,425.0},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,475.0,475.0,475.0,475.0},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,550.0,550.0}} //slot 12, chain
  },
  {//crate 3
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,400.0,400.0,400.0}},//slot 8, chain  
    {{412.5,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,437.5,437.5,437.5},{475.0,475.0,462.5,450.0,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,475.0,475.0},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.0,525.0,512.5,512.5,512.5},{550.0,550.0,550.0,537.5,537.5}} //slot 12, chain
  },
  {//crate 4
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 5
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain     
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{500.0,512.5,512.5,512.5,487.5}},//slot 11, chain  
    {{525.0,525.0,512.5,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  }, 
  {//crate 6
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  }, 
  {//crate 7
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain     
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{500.0,512.5,512.5,512.5,487.5}},//slot 11, chain  
    {{525.0,525.0,512.5,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 8
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 9
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 10
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  }, 
  {//crate 11
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 12
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 13
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{362.5,362.5,350.0,350.0,350.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 14
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  }, 
  {//crate 15
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{362.5,362.5,350.0,350.0,350.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 16
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 17
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 18
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 19
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 20
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 21
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 22
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 23
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 24
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,425.0,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,437.5,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain 
  },
  {//crate 25
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 26
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,425.0,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,437.5,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain 
  },
  {//crate 27
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 28
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain 
  },
  {//crate 29
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,512.5,525.0,525.0},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain   
  },
  {//crate 30
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 31
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,512.5,525.0,525.0},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 32
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain 
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,387.5,387.5,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,387.5,400.0},{412.5,412.5,425.0,412.5,425.0}},//slot 9, chain
    {{425.0,425.0,425.0,425.0,425.0},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,462.5,462.5,475.0,475.0},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 33
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{400.0,400.0,400.0,400.0,400.0}},//slot 8, chain  
    {{412.5,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{437.5,437.5,437.5,437.5,437.5},{462.5,462.5,462.5,450.0,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,475.0,475.0},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.5,512.5,512.5,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 34
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,387.5,387.5,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,387.5,400.0},{412.5,412.5,425.0,412.5,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,425.0,425.0},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,462.5,462.5,475.0,475.0},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 35
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{400.0,400.0,400.0,400.0,400.0}},//slot 8, chain  
    {{412.5,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{437.5,437.5,437.5,437.5,437.5},{462.5,462.5,462.5,450.0,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,475.0,475.0},{512.5,512.5,512.5,500.0,500.0}},//slot 11, chain  
    {{525.5,512.5,512.5,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 36
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{512.5,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 37
     {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain
     {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain
     {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain
     {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain
     {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain
     {{387.5,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain
     {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain
     {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain
     {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,512.5}},//slot 11, chain
     {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 38
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{512.5,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 39
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{387.5,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,512.5}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 40
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 41
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,326.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,387.5},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 42
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 43
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,326.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,387.5},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 44
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 45
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 46
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,487.5,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 47
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  }, 
  {//crate 48
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{400.0,400.0,412.5,412.5,412.5}},//slot 9, chain  
    {{437.5,437.5,437.5,437.5,437.5},{462.5,462.5,462.5,462.5,462.5}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 49
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,400.0,400.0,400.0,400.0}},//slot 8, chain  
    {{437.5,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,437.5,437.5,437.5},{475.0,475.0,475.0,475.0,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 50
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,412.5}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,450.0,450.0}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{487.5,487.5,487.5,500.0,500.0}},//slot 11, chain  
    {{500.0,500.0,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 51
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,412.5}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,425.0,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,437.5,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,512.5,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 52
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,475.0,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 53
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 54
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,475.0,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 55
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
    {//crate 56
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,462.5,475.0,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 57
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,462.5,462.5,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
 {//crate 58
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{462.5,462.5,475.0,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 59
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,425.0,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,462.5,462.5,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
   {//crate 60
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain     
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 61
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 62
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain     
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,437.5,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,0,0},{487.5,487.5,487.5,0,0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 63
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,425.0,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{0,0,0,487.5,487.5},{0,0,0,512.5,500.0}},//slot 11, chain  
    {{0,0,0,0,0},{0,0,0,0,0}} //slot 12, chain
  },
  {//crate 64
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 65
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,387.5},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 66
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,400.0,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,400.0,400.0,412.5},{412.5,412.5,425.0,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{475.0,450.0,450.0,462.5,462.5}},//slot 10, chain  
    {{475.0,475.0,475.0,487.5,487.5},{500.0,500.0,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 67
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,387.5},{412.5,412.5,412.5,400.0,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,437.5}},//slot 9, chain  
    {{450.0,450.0,450.0,450.0,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 68
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain
  },
  {//crate 69
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  },
  {//crate 70
    {{0,0,0,0,0},{0,0,0,0,0}},//slot 3, chain  
    {{175.0,175.0,175.0,187.5,187.5},{200.0,200.0,212.5,212.5,212.5}},//slot 4, chain  
    {{212.5,237.5,225.0,237.5,237.5},{250.0,262.5,262.5,262.5,262.5}},//slot 5, chain  
    {{262.5,262.5,275.0,275.0,275.0},{287.5,287.5,300.0,300.0,300.0}},//slot 6, chain  
    {{312.5,300.0,312.5,312.5,325.0},{337.5,325.0,337.5,337.5,350.0}},//slot 7, chain  
    {{350.0,362.5,350.0,362.5,362.5},{387.5,387.5,400.0,400.0,400.0}},//slot 8, chain  
    {{387.5,387.5,387.5,400.0,400.0},{412.5,412.5,412.5,425.0,425.0}},//slot 9, chain  
    {{425.0,425.0,425.0,437.5,437.5},{462.5,450.0,450.0,462.5,450.0}},//slot 10, chain  
    {{475.0,475.0,487.5,487.5,487.5},{487.5,487.5,500.0,500.0,500.0}},//slot 11, chain  
    {{500.0,512.5,512.5,512.5,512.5},{525.0,537.5,537.5,537.5,550.0}} //slot 12, chain    
  },
  {//crate 71
    {{162.5,0,0,0,0},{175.0,0,0,0,0}},//slot 3, chain  
    {{187.5,200.0,200.0,187.5,187.5},{212.5,225.0,225.0,212.5,212.5}},//slot 4, chain  
    {{250.0,250.0,237.5,237.5,237.5},{275.0,275.0,275.0,262.5,262.5}},//slot 5, chain  
    {{287.5,287.5,287.5,275.0,275.0},{312.5,312.5,312.5,300.0,300.0}},//slot 6, chain  
    {{337.5,337.5,325.0,325.0,325.0},{362.5,362.5,350.0,350.0,350.0}},//slot 7, chain  
    {{375.0,375.0,375.0,375.0,375.0},{412.5,412.5,412.5,412.5,400.0}},//slot 8, chain  
    {{425.0,412.5,412.5,412.5,412.5},{437.5,437.5,437.5,437.5,425.0}},//slot 9, chain  
    {{450.0,450.0,450.0,437.5,437.5},{475.0,475.0,475.0,462.5,462.5}},//slot 10, chain  
    {{487.5,487.5,487.5,487.5,487.5},{512.5,512.5,512.5,512.5,500.0}},//slot 11, chain  
    {{525.0,525.0,525.0,525.0,512.5},{550.0,550.0,537.5,537.5,537.5}} //slot 12, chain
  }
};
    
//_____________________________________________________________________________
AliTOFCableLengthMap::AliTOFCableLengthMap(){
  //
  // AliTOFCableLengthMap default constructor
  //
  
}
//_____________________________________________________________________________
AliTOFCableLengthMap::~AliTOFCableLengthMap(){
  //
  // AliTOFCableLengthMap destructor
  //
}
//_____________________________________________________________________________
Float_t  AliTOFCableLengthMap::GetCableLength(Int_t icrate, Int_t islot, Int_t ichain, Int_t itdc)  {
  //
  //   return the length of the cable (in cm) that comes out from:
  //   crate# icrate,
  //   slot# islot,
  //   chain# ichain,
  //   tdc# itdc 
  //
  Int_t dummyTdc= itdc / 3;
  Int_t dummySlot= islot - 3;
  
  return  fgkCableLength[icrate][dummySlot][ichain][dummyTdc];
}
//_____________________________________________________________________________
Float_t  AliTOFCableLengthMap::GetCableTimeShift(Int_t icrate, Int_t islot, Int_t ichain, Int_t itdc) {
  //
  //   return the time shift (in ns) due to cable 
  //   according its length:
  //   it's 5.13 -measured- (4.35 nominal) ns per meter
  //   that is 0.0513 (0.0435) ns per cm
  //
 
  Int_t dummyTdc= itdc / 3;
  Int_t dummySlot= islot - 3;
  
  //Float_t cableTimeShift = fgkCableLength[icrate][dummySlot][ichain][dummyTdc] * 0.0435; // nominal
  //Float_t cableTimeShift = fgkCableLength[icrate][dummySlot][ichain][dummyTdc] * 0.0513; // measured
  Float_t cableTimeShift = fgkCableLength[icrate][dummySlot][ichain][dummyTdc] * fgkPropagationDelay;

  return  cableTimeShift;
}

//_____________________________________________________________________________

Int_t AliTOFCableLengthMap::GetCableTimeShiftBin(Int_t icrate, Int_t islot, Int_t ichain, Int_t itdc)
{
  //
  //   return the time shift (in bin number) due to cable 
  //   according its length:
  //   it's 5.13 -measured- (4.35 nominal) ns per meter
  //   that is 0.0513 (0.0435) ns per cm
  //

  Float_t timeNS = GetCableTimeShift(icrate, islot, ichain, itdc);
  timeNS*=1000.;
  timeNS/=AliTOFGeometry::TdcBinWidth();

  Int_t dummy = (Int_t)(timeNS);

  //Int_t dummy = (Int_t)(GetCableTimeShift(icrate, islot, ichain, itdc)*1000./AliTOFGeometry::TdcBinWidth());

  return dummy;

}
